#!/usr/bin/env python3

import os
import sys
import site
import obt.path
import importlib.metadata
from obt.deco import Deco
deco = Deco()

# Column width constants
COL_FOLDER = 28
COL_STATE = 10
COL_BRANCH = 20
COL_HASH = 14
COL_MESSAGE = 40

def print_env_var(name, default):
    value = os.getenv(name, default)
    print(f"{deco.key(name)}: {deco.val(value)}")

def print_item(name, value):
    print(f"{deco.key(name)}: {deco.val(value)}")

print( "######################################################")

print_env_var(f'PYTHONPATH', sys.path)
print_env_var(f'PYTHONHOME', sys.prefix)
print_env_var(f'PYTHONSTARTUP', 'Not set')
print_env_var(f'PYTHONUSERBASE', site.USER_BASE)
print_env_var(f'PYTHONEXECUTABLE', sys.executable)
print_env_var(f'PYTHONWARNINGS', 'Not set')
print_env_var(f'PYTHONNOUSERSITE', 'Not set (User site directory is added to sys.path)')
print_env_var(f'PYTHONUNBUFFERED', 'Not set (Buffered I/O is used for stdout and stderr)')

print_env_var(f'site.PREFIXES', site.PREFIXES)
print_env_var(f'site.USER_SITE', site.USER_SITE)
print_env_var(f'site.USER_BASE', site.USER_BASE)
print_env_var(f'sys.prefix', sys.prefix)
print_env_var(f'sys.base_prefix', sys.base_prefix)

print( "######################################################")

a = importlib.metadata.distribution("ork.build").metadata

print_item( "obt-pymodule-path",obt.path.obt_module_path() )
print_item( "obt-data-base",obt.path.obt_data_base() )
print_item( "obt-modules-test",obt.path.__get_modules() )
print_item( "obt-test-inplace",obt.path.__is_inplace() )
print_item( "obt-modules-base",obt.path.obt_modules_base() )
print_item( "running_from_pip",obt.path.running_from_pip() )
print_item( "running_in_tree",obt.path.obt_in_tree() )
print_item( "obt.distrib.name",a["Name"] )
print_item( "obt.distrib.author",a["Author"] )
print_item( "obt.distrib.author-email",a["Author-email"] )
print_item( "obt.distrib.summary",a["Summary"] )
print_item( "obt.distrib.homepage",a["Home-page"] )
print( "################################################")
print_item( "obt.distrib.version",a["Version"] )


plist = os.environ.get("OBT_PROJECTS_LIST", "")
project_list = plist.split(":") if plist else []

def get_git_info():
  import git
  try:
    repo = git.Repo(search_parent_directories=True)
    hash = repo.head.object.hexsha
    is_dirty = repo.is_dirty(untracked_files=True)
    
    # Get current branch name
    try:
        branch = repo.active_branch.name
    except TypeError:
        # HEAD is detached
        branch = "detached HEAD"
    
    # Get commit message (first line only)
    message = repo.head.object.message.split('\n')[0]
    
    return hash, is_dirty, branch, message
  except git.InvalidGitRepositoryError:
    return None, None, None, None

def truncate_with_ellipsis(text, max_width):
    """Truncate text to fit within max_width, adding ellipsis if needed"""
    if len(text) > max_width:
        return text[:max_width-3] + "..."
    return text

def reverse_video(text):
    """Apply reverse video (swap foreground/background) to text"""
    return f"\033[7m{text}\033[0m"

def center_text(text, width):
    """Center text within given width"""
    return text.center(width)

# Print header if there are git projects
if project_list and any(obt.path.Path(item).exists() for item in project_list):
    print( "######################################################")
    print(deco.key("Git Repositories:"))
    print()
    
    # Print reverse video header with centered column labels
    header_row = (
        center_text("REPOSITORY", COL_FOLDER - 6).rjust(COL_FOLDER) +
        center_text("STATE", COL_STATE) +
        center_text("BRANCH", COL_BRANCH) +
        center_text("COMMIT", COL_HASH) +
        center_text("MESSAGE", COL_MESSAGE)
    )
    print(reverse_video(header_row))

for item in project_list:
  item = obt.path.Path(item)
  if item.exists():
    os.chdir(item)
    git_hash, is_dirty, branch, message = get_git_info()
    if git_hash:
      # Prepare folder name - check if it matches any environment variable
      folder_name = str(item)
      
      # Check all environment variables for exact match, excluding common ones
      env_var_name = None
      excluded_vars = {'PWD', 'OLDPWD', 'HOME', 'PATH', 'SHELL', 'USER', 'LOGNAME', 
                       'TERM', 'LANG', 'LC_ALL', 'LC_CTYPE', 'DISPLAY', 'EDITOR',
                       'VISUAL', 'TMPDIR', 'TMP', 'TEMP', 'HOSTNAME', 'PS1', 'PS2',
                       'PYTHONPATH', 'PYTHONHOME', 'PYTHONSTARTUP', 'PYTHONUSERBASE',
                       'PYTHONEXECUTABLE', 'PYTHONWARNINGS', 'PYTHONNOUSERSITE',
                       'PYTHONUNBUFFERED', '_'}
      
      for key, value in os.environ.items():
        if key not in excluded_vars and value == folder_name:
          env_var_name = key
          break
      
      # Use env var name if found, otherwise use folder path
      if env_var_name:
        folder_display = truncate_with_ellipsis(f"${env_var_name}", COL_FOLDER - 1)
      else:
        folder_display = truncate_with_ellipsis(folder_name, COL_FOLDER - 1)
      
      # Prepare branch name
      branch_display = truncate_with_ellipsis(branch, COL_BRANCH - 1)
      
      # Prepare message
      message_display = truncate_with_ellipsis(message, COL_MESSAGE - 1)
      
      # Color elements (reordered: folder, state, branch, hash, message)
      # Right-justify repo folder
      colored_folder = deco.val(folder_display.rjust(COL_FOLDER))
      
      # Color and center status
      if is_dirty:
        status_text = "modified"
        colored_status = deco.red(status_text.center(COL_STATE))
      else:
        status_text = "clean"
        colored_status = deco.rgbstr(0, 255, 0, status_text.center(COL_STATE))
      
      # Center branch and hash
      colored_branch = deco.rgbstr(191, 191, 191, branch_display.center(COL_BRANCH))
      colored_hash = deco.rgbstr(255, 179, 51, git_hash[:8].center(COL_HASH))
      
      # Left-justify message (already left-justified by default)
      colored_message = deco.rgbstr(128, 192, 255, message_display.ljust(COL_MESSAGE))
      
      # Print formatted line (reordered columns)
      print(f"{colored_folder}{colored_status}{colored_branch}{colored_hash}{colored_message}")
    else:
      print(f"{deco.val(str(item).ljust(COL_FOLDER))}{deco.red('not a repo')}")


###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, string

###############################################################################

# Color database - RGB values for named colors
COLOR_DB = {
    # Basic ANSI colors
    'black': (0, 0, 0),
    'red': (255, 0, 0),
    'green': (0, 255, 0),
    'yellow': (255, 255, 0),
    'blue': (0, 0, 255),
    'magenta': (255, 0, 255),
    'cyan': (0, 255, 255),
    'white': (255, 255, 255),
    
    # Bright variants
    'brightred': (255, 128, 128),
    'brightgreen': (128, 255, 128),
    'brightblue': (128, 128, 255),
    
    # Extended colors
    'orange': (255, 128, 0),
    'purple': (128, 0, 255),
    'pink': (255, 192, 203),
    
    # Grey levels using dedicated greyscale ramp (232-255) - all 24 levels
    # Each step is +10 in RGB values: grey0=RGB(8,8,8) to grey23=RGB(238,238,238)
    'grey0': 232,    # RGB(8,8,8)
    'grey1': 233,    # RGB(18,18,18)
    'grey2': 234,    # RGB(28,28,28)
    'grey3': 235,    # RGB(38,38,38)
    'grey4': 236,    # RGB(48,48,48)
    'grey5': 237,    # RGB(58,58,58)
    'grey6': 238,    # RGB(68,68,68)
    'grey7': 239,    # RGB(78,78,78)
    'grey8': 240,    # RGB(88,88,88)
    'grey9': 241,    # RGB(98,98,98)
    'grey10': 242,   # RGB(108,108,108)
    'grey11': 243,   # RGB(118,118,118)
    'grey12': 244,   # RGB(128,128,128) - Middle grey
    'grey13': 245,   # RGB(138,138,138)
    'grey14': 246,   # RGB(148,148,148)
    'grey15': 247,   # RGB(158,158,158)
    'grey16': 248,   # RGB(168,168,168)
    'grey17': 249,   # RGB(178,178,178)
    'grey18': 250,   # RGB(188,188,188)
    'grey19': 251,   # RGB(198,198,198)
    'grey20': 252,   # RGB(208,208,208)
    'grey21': 253,   # RGB(218,218,218)
    'grey22': 254,   # RGB(228,228,228)
    'grey23': 255,   # RGB(238,238,238)
    
    # Aliases
    'gray': (128, 128, 128),
    'gray0': (32, 32, 32),
    'gray1': (64, 64, 64),
    'gray2': (96, 96, 96),
    'gray3': (128, 128, 128),
    'gray4': (160, 160, 160),
    'gray5': (192, 192, 192),
    'gray6': (224, 224, 224),
    'gray7': (240, 240, 240),
}

###############################################################################

class Theme:
  def __init__(self,bash):
    super().__init__()
    self.bash = bash
  ###############################
  def rgb256(self,r,g,b):
    r = int((r*5)/255)
    g = int((g*5)/255)
    b = int((b*5)/255)
    color = 16 + 36 * r + 6 * g + b
    rval = "\033[38;5;%dm" % color
    if self.bash:
      rval = "\\[" + rval + "\\]"
    return rval
  ###############################
  def blink(self):
    return "\033[5m"
  ###############################
  def vrgb256(self,r,g,b):
    return self.rgb256(r,g,b)
  ###############################
  def rgbstr(self,r,g,b,string):
    return self.rgb256(r,g,b)+str(string)+self.reset()
  ###############################
  def reset(self):
    rval = "\033[m"
    if self.bash:
      rval = "\\["+rval+"\\]"
    return rval
  ###############################
  def key(self,string):
    return self.vrgb256(255,255,0)+str(string)+self.reset()
  def val(self,string):
    return self.vrgb256(255,255,255)+str(string)+self.reset()
  def path(self,string):
    return self.vrgb256(255,255,128)+str(string)+self.reset()
  def inf(self,string):
    return self.vrgb256(128,128,255)+str(string)+self.reset()
  def warn(self,string):
    return self.yellow(string)+self.reset()
  def err(self,string):
    return self.red(string)+self.reset()
  ###############################
  def bright(self,string):
    return self.vrgb256(255,255,255)+str(string)+self.reset()
  ###############################
  def red(self,string):
    return self.vrgb256(255,0,0)+str(string)+self.reset()
  def green(self,string):
    return self.vrgb256(0,255,0)+str(string)+self.reset()
  def yellow(self,string):
    return self.vrgb256(255,255,0)+str(string)+self.reset()
  def blue(self,string):
    return self.vrgb256(0,0,255)+str(string)+self.reset()
  def magenta(self,string):
    return self.vrgb256(255,0,255)+str(string)+self.reset()
  def cyan(self,string):
    return self.vrgb256(0,255,255)+str(string)+self.reset()
  def white(self,string):
    return self.vrgb256(255,255,255)+str(string)+self.reset()
  def orange(self,string,blink=False):
    bl = self.blink() if blink else ""
    return self.vrgb256(255,128,0)+bl+str(string)+self.reset()
  ###############################
  def reverseVideo(self,string):
    """Apply reverse video (swap foreground/background colors)"""
    rval = "\033[7m"
    if self.bash:
      rval = "\\[" + rval + "\\]"
    return rval + str(string) + self.reset()
  ###############################
  def promptL(self,string):
    return self.vrgb256(255,0,0)+str(string)+self.reset()
  ###############################
  def promptC(self,string):
    return self.vrgb256(255,255,0)+str(string)+self.reset()
  ###############################
  def promptR(self,string):
    return self.vrgb256(255,128,0)+str(string)+self.reset()
  ###############################

###############################################################################

class DarkTheme(Theme):
  def __init__(self,bash=False):
    super().__init__(bash=bash)
  ###############################
  def val(self,string):
    return self.vrgb256(255,192,192)+str(string)+self.reset()
  ###############################
  def bright(self,string):
    return self.vrgb256(255,255,128)+str(string)+self.reset()

###############################################################################

class InverseTheme(Theme):
  def __init__(self,bash=False):
    super().__init__(bash=bash)
  ###############################
  def vrgb256(self,r,g,b):
    return self.rgb256(255-r,255-g,255-b)
  ###############################
  def promptL(self,string):
    return self.rgb256(255,0,0)+str(string)+self.reset()
  ###############################
  def promptC(self,string):
    return self.rgb256(255,255,0)+str(string)+self.reset()
  ###############################
  def promptR(self,string):
    return self.rgb256(255,128,0)+str(string)+self.reset()
  #def rgbstr(self,r,g,b,string):
  #  return super().rgb256(255-r,255-g,255-b)+str(string)+self.reset()

###############################################################################

class LightTheme(Theme):
  def __init__(self,bash=False):
    super().__init__(bash=bash)
  ###############################
  def key(self,string):
    return self.rgb256(25,25,0)+str(string)+self.reset()
  def val(self,string):
    return self.rgb256(0,160,160)+str(string)+self.reset()
  def path(self,string):
    return self.rgb256(65,65,65)+str(string)+self.reset()
  def inf(self,string):
    return self.rgb256(15,15,30)+str(string)+self.reset()
  def warn(self,string):
    return self.yellow(string)+self.reset()
  def err(self,string):
    return self.red(string)+self.reset()
  ###############################
  def bright(self,string):
    return self.rgb256(0,0,0)+str(string)+self.reset()
  ###############################
  def vrgb256(self,r,g,b):
    return self.rgb256(255-r,255-g,255-b)
  ###############################
  def promptL(self,string):
    return self.rgb256(128,0,0)+str(string)+self.reset()
  ###############################
  def promptC(self,string):
    return self.rgb256(128,128,0)+str(string)+self.reset()
  ###############################
  def promptR(self,string):
    return self.rgb256(128,96,0)+str(string)+self.reset()

###############################################################################

class MonoTheme(Theme):
  def __init__(self,bash=False):
    super().__init__(bash=bash)
  ###############################
  def rgb256(self,r,g,b):
    return ""
  ###############################
  def reset(self):
    return ""

###############################################################################

class Deco:
  
  ###############################
  def __init__(self,bash=False):
    if "OBT_THEME" in os.environ:
      if os.environ["OBT_THEME"]=="dark":
        self._theme = DarkTheme(bash=bash)
      elif os.environ["OBT_THEME"]=="light":
        self._theme = LightTheme(bash=bash)
      elif os.environ["OBT_THEME"]=="inverse":
        self._theme = InverseTheme(bash=bash)
      elif os.environ["OBT_THEME"]=="mono":
        self._theme = MonoTheme(bash=bash)
    else:
      self._theme = DarkTheme(bash=bash)
  ###############################
  def rgbstr(self,r,g,b,string):
    return self._theme.rgbstr(r,g,b,string)
  ###############################
  def magenta(self,string):
    return self._theme.magenta(string)
  def cyan(self,string):
    return self._theme.cyan(string)
  def white(self,string):
    return self._theme.white(string)
  def orange(self,string,blink=False):
    return self._theme.orange(string,blink=blink)
  def yellow(self,string):
    return self._theme.yellow(string)
  def red(self,string):
    return self._theme.red(string)
  def green(self,string):
    return self._theme.green(string)
  def blue(self,string):
    return self._theme.blue(string)
  def black(self,string):
    return self._theme.black(string) if hasattr(self._theme, 'black') else string
  def gray(self,string):
    return self._theme.gray(string) if hasattr(self._theme, 'gray') else self._theme.rgbstr(128,128,128,string)
  def brightred(self,string):
    return self._theme.brightred(string) if hasattr(self._theme, 'brightred') else self._theme.rgbstr(255,128,128,string)
  def brightgreen(self,string):
    return self._theme.brightgreen(string) if hasattr(self._theme, 'brightgreen') else self._theme.rgbstr(128,255,128,string)
  def brightblue(self,string):
    return self._theme.brightblue(string) if hasattr(self._theme, 'brightblue') else self._theme.rgbstr(128,128,255,string)
  def purple(self,string):
    return self._theme.purple(string) if hasattr(self._theme, 'purple') else self._theme.rgbstr(128,0,255,string)
  def pink(self,string):
    return self._theme.pink(string) if hasattr(self._theme, 'pink') else self._theme.rgbstr(255,192,203,string)
  # Grey levels (0=darkest, 7=lightest)
  def grey0(self,string):  # Near black
    return self._theme.grey0(string) if hasattr(self._theme, 'grey0') else self._theme.rgbstr(32,32,32,string)
  def grey1(self,string):
    return self._theme.grey1(string) if hasattr(self._theme, 'grey1') else self._theme.rgbstr(64,64,64,string)
  def grey2(self,string):
    return self._theme.grey2(string) if hasattr(self._theme, 'grey2') else self._theme.rgbstr(96,96,96,string)
  def grey3(self,string):
    return self._theme.grey3(string) if hasattr(self._theme, 'grey3') else self._theme.rgbstr(128,128,128,string)
  def grey4(self,string):
    return self._theme.grey4(string) if hasattr(self._theme, 'grey4') else self._theme.rgbstr(160,160,160,string)
  def grey5(self,string):
    return self._theme.grey5(string) if hasattr(self._theme, 'grey5') else self._theme.rgbstr(192,192,192,string)
  def grey6(self,string):
    return self._theme.grey6(string) if hasattr(self._theme, 'grey6') else self._theme.rgbstr(224,224,224,string)
  def grey7(self,string):  # Near white
    return self._theme.grey7(string) if hasattr(self._theme, 'grey7') else self._theme.rgbstr(240,240,240,string)
  ###############################
  def key(self,string):
    return self._theme.key(string)
  def val(self,string):
    return self._theme.val(string)
  def path(self,string):
    return self._theme.path(string)
  def inf(self,string):
    return self._theme.inf(string)
  def warn(self,string):
    return self._theme.warn(string)
  def err(self,string):
    return self._theme.err(string)
  ###############################
  def bright(self,string):
    return self._theme.bright(string)
  ###############################
  def promptL(self,string):
    return self._theme.promptL(string)
  ###############################
  def promptC(self,string):
    return self._theme.promptC(string)
  ###############################
  def promptR(self,string):
    return self._theme.promptR(string)
  ###############################
  def reverseVideo(self,string):
    return self._theme.reverseVideo(string)
  ###############################
  def fg(self, color):
    """Return ANSI escape sequence for foreground color by name, RGB tuple, or color index"""
    if isinstance(color, str):
      if color in COLOR_DB:
        value = COLOR_DB[color]
        if isinstance(value, int):
          # Direct color index
          return "\033[38;5;%dm" % value
        else:
          # RGB tuple
          r, g, b = value
      else:
        return ""  # Unknown color
    elif isinstance(color, int):
      # Direct color index (0-255)
      return "\033[38;5;%dm" % color
    elif isinstance(color, (tuple, list)) and len(color) == 3:
      r, g, b = color
    else:
      return ""
    
    # Convert RGB to 256 color palette
    r = int((r * 5) / 255)
    g = int((g * 5) / 255)
    b = int((b * 5) / 255)
    color_code = 16 + 36 * r + 6 * g + b
    return "\033[38;5;%dm" % color_code
  ###############################
  def bg(self, color):
    """Return ANSI escape sequence for background color by name, RGB tuple, or color index"""
    if isinstance(color, str):
      if color in COLOR_DB:
        value = COLOR_DB[color]
        if isinstance(value, int):
          # Direct color index
          return "\033[48;5;%dm" % value
        else:
          # RGB tuple
          r, g, b = value
      else:
        return ""  # Unknown color
    elif isinstance(color, int):
      # Direct color index (0-255)
      return "\033[48;5;%dm" % color
    elif isinstance(color, (tuple, list)) and len(color) == 3:
      r, g, b = color
    else:
      return ""
    
    # Convert RGB to 256 color palette
    r = int((r * 5) / 255)
    g = int((g * 5) / 255)
    b = int((b * 5) / 255)
    color_code = 16 + 36 * r + 6 * g + b
    return "\033[48;5;%dm" % color_code
  ###############################
  def reset(self):
    """Return ANSI reset sequence"""
    return "\033[m"
  ###############################
  def strip_ansi(self, text):
    """Remove ANSI escape sequences from text to get visible length"""
    import re
    ansi_escape = re.compile(r'\x1b\[[0-9;]*m')
    return ansi_escape.sub('', text)
  ###############################
  def visible_len(self, text):
    """Get the visible length of text (excluding ANSI codes)"""
    return len(self.strip_ansi(text))
  ###############################
  def formatColumns(self, column_configs, column_texts):
    """
    Format text into aligned columns, handling ANSI escape codes properly.
    
    Args:
      column_configs: List of tuples (width, alignment) where:
        - width: int or None (None means no padding)
        - alignment: 'left', 'right', or 'center'
      column_texts: List of strings (may contain ANSI codes)
    
    Returns:
      Formatted string with proper column alignment
    """
    formatted_cols = []
    
    for i, (text, config) in enumerate(zip(column_texts, column_configs)):
      if config is None or len(config) < 1:
        # No formatting for this column
        formatted_cols.append(text)
        continue
      
      width = config[0] if len(config) > 0 else None
      align = config[1] if len(config) > 1 else 'left'
      
      if width is None:
        # No width specified, just use the text as-is
        formatted_cols.append(text)
      else:
        # Calculate visible length and padding needed
        visible_length = self.visible_len(text)
        padding_needed = width - visible_length
        
        if padding_needed <= 0:
          # Text is already longer than width, just use it
          formatted_cols.append(text)
        else:
          # Apply alignment with proper padding
          if align == 'right':
            formatted_cols.append(' ' * padding_needed + text)
          elif align == 'center':
            left_pad = padding_needed // 2
            right_pad = padding_needed - left_pad
            formatted_cols.append(' ' * left_pad + text + ' ' * right_pad)
          else:  # left
            formatted_cols.append(text + ' ' * padding_needed)
    
    return ''.join(formatted_cols)

###############################################################################

class DecoFormatter(string.Formatter):
  def __init__(self):
    self.deco = Deco()    
  def format_field(self, value, format_spec):
    if isinstance(value, str):
      if format_spec.endswith('red'):
        value = self.deco.red(value)
        format_spec = format_spec[:-3]
    return super(DecoFormatter, self).format(value, format_spec)

###############################################################################

__all__ =	[ "deco" ]



#!/usr/bin/env python3
import jinja2, git, datetime, os, subprocess, argparse, shlex, logging, sys, threading, time
from http import server
from yarl import URL
from pathlib import Path
import badge, uuid, urllib, scheduler, httphandler, orkbb

SCRIPTDIR=Path(os.path.dirname(os.path.realpath(__file__)))

################################################################################

parser = argparse.ArgumentParser(description='orkbb-master')
parser.add_argument('--project', metavar="projectfile", required=True, help='path of json project file' )
parser.add_argument('--outputdir', default=SCRIPTDIR/"output", metavar="outputdir", help='outputdirectory' )
parser.add_argument("--debug",action="store_true")
parser.add_argument("--verbose",action="store_true")
args = vars(parser.parse_args())

################################################################################

VERBOSE=args["verbose"]
OUTPUTDIR = Path(args["outputdir"])
print("remove old outputdir")
os.system("rm -rf %s"%OUTPUTDIR)
os.system("mkdir -p %s"%OUTPUTDIR)

################################################################################
# scheduler
################################################################################

projectpath = Path(args["project"])
sched = scheduler.Scheduler(projectpath,OUTPUTDIR)
#print(SCHED)
http_addr = sched.config.http_bind_addr
http_port = sched.config.http_bind_port
################################################################################
# web server
################################################################################
os.chdir(OUTPUTDIR)
server_address = (http_addr, http_port)
handler = httphandler.HandlerClass(OUTPUTDIR,sched.sitelock)
httpd = server.HTTPServer(server_address, handler)
httpd.serve_forever()



#!/usr/bin/env python3
###############################################################################
import sys, os, subprocess
from obt import host, path, pathtools
from obt.command import capture, run
from obt.command import Deco
deco = Deco()
###############################################################################
def find_replace(inpstring, dictionary):
  print(dictionary)
  for item in inpstring:
    if item in dictionary.keys():
      print("replacing item<%s>"%item)
      inpstring = inpstring.replace(item, dictionary[item])
  return inpstring

###############################################################################

def macho_enumerate_dylibs(mach_o_path):
  assert (host.IsOsx)
  loadlines = capture(["otool","-l",mach_o_path],do_log=False).splitlines()
  cmdindex = 0
  cmd = None
  name = None
  state = 0
  dylib_paths = list()
  for line in loadlines:
    tokens = line.split(" ")
    tokens = [i for i in tokens if i]
    if state==0:
      if tokens[0]=="Load" and tokens[1]=="command":
        cmdindex = int(tokens[2])
        state = 1
    elif state==1:
      if tokens[0]=="cmd" and tokens[1]=="LC_LOAD_DYLIB":
        cmd = "LC_LOAD_DYLIB"
        state = 2
    elif state==2:
      if tokens[0] == "name":
        dylib_paths += [tokens[1]]
        state=0
    else:
      assert(False)
  return dylib_paths

###############################################################################

def macho_get_all_dylib_dependencies(file_path, seen=None):
  """Recursively get a list of dynamic library dependencies for the file."""
  if seen is None:
    seen = set()
  if not os.path.exists(file_path) or not os.path.isfile(file_path):
    return seen
  deps = subprocess.check_output(['otool', '-L', file_path]).decode()
  for line in deps.splitlines()[1:]:
    dylib = line.split()[0]
    if dylib not in seen:
      seen.add(dylib)
      macho_get_all_dylib_dependencies(dylib, seen)
  return seen

###############################################################################

def macho_replace_loadpaths(mach_o_path,search,replace):
  assert (host.IsOsx)
  str_search = str(search)
  str_replace = str(replace)
  dylib_paths = macho_enumerate_dylibs(mach_o_path)
  for inpitem in dylib_paths:
    outitem = inpitem.replace(str_search, str_replace)
    #print("mach_o_path: " + str(mach_o_path) + " inp: " + inpitem + " search: " + str_search + " replace: " + str_replace)
    if outitem!=inpitem:
      run(["install_name_tool","-change",inpitem,outitem,mach_o_path],do_log=False)

###############################################################################

def macho_change_id(mach_o_path,id_str):
  assert (host.IsOsx)
  run(["install_name_tool","-id",'%s'%id_str,mach_o_path],do_log=False)

###############################################################################

def macho_get_id(mach_o_path):
  assert (host.IsOsx)
  return capture(["otool","-D",mach_o_path],do_log=False)

##############################################################################

def macho_dump(mach_o_path):
  print(deco.val("/////////////////////////////////////////////////////////"))
  print(deco.val("MachO Dump: ") + deco.val(mach_o_path))
  print(deco.val("/////////////////////////////////////////////////////////"))
  dylib_paths = macho_enumerate_dylibs(mach_o_path)
  for item in dylib_paths:
    print(deco.val("dylib: ")+deco.path(item))
  print(deco.val("/////////////////////////////////////////////////////////"))

##############################################################################

def enumerateOrkLibs(basepath):
  return pathtools.recursive_patglob(basepath,"libork*.dylib")

##############################################################################

def enumerateOrkPyMods(basepath):
  return pathtools.recursive_patglob(basepath,"*.so")

##############################################################################

class DylibReference(object):
  def __init__(self):
    self.references = set()

class DylibReferenceDatabase(object):
  def __init__(self):
    self.referencers = dict()
    self.references = set()
  def probe(self,dylib_list):
    for item in dylib_list:
      deps = macho_enumerate_dylibs(item)
      for dep in deps:
        if str(homebrew_dir) in dep:
          key = str(item)
          print(key)
          self.referencers.setdefault(key, DylibReference()).references.add(dep)
          self.references.add(dep)
  def probe_in(self,directory,dylib_list):
    for item in dylib_list:
      deps = macho_enumerate_dylibs(item)
      for dep in deps:
        if str(directory) in dep:
          key = str(item)
          print(key)
          self.referencers.setdefault(key, DylibReference()).references.add(dep)
          self.references.add(dep)
      #macho_dump(item)
  def dump_referencers(self):
    for item in self.referencers:
      print(item)
  def dump_references(self):
    for item in self.references:
      print(item)
